FFITests
 subclass: #FFIBasicTests
 instanceVariableNames: ''
 classVariableNames: ' '
 poolDictionaries: ''
 category: 'FFIBasicTests'!
FFIBasicTests comment: '
	Minimal set of examples and informal tests of FFI framework for S8.
'!

! FFIBasicTests class methodsFor: 'tests' !
test00
	" Try to access ffi & basic support. "

	^#Ok ,' - Running ',self name! !

! FFIBasicTests class methodsFor: 'tests' !
test01
	" Try to access ffi & basic support. "

	| ffi pointer |
	ffi := self ffi. ffi isNil ifTrue: [ ^self error: 'FFI module not found.' ].
	pointer := ffi basicAt: #Pointer. pointer isNil ifTrue: [ ^self error: 'Pointer support not found.' ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test02
	" Try to instantiate a pointer to external memory. "

	| ptr |
	ptr := self newPointer: 1024.
	^#Ok ,' - Pointer allocated at ', (ptr#address #toString: 16) asUppercase! !

! FFIBasicTests class methodsFor: 'tests' !
test03
	" Seek a pointer. "

	| ptr1 ptr2 delta |
	delta := 32.
	ptr1 := self newPointer: 10.
	ptr2 := ptr1 #seek: delta.
	ptr1#address + delta = ptr2#address ifTrue: [ ^#Ok ].
	^self error: 'Seek failed prt1 at ',ptr1#address ,' ptr2 at ',ptr2#address ,' delta=',delta! !

! FFIBasicTests class methodsFor: 'tests' !
test04

	| ptr k |
	ptr := self newPointer: 10.

	self throws: [ ptr #putInt8: (self math: 2 pow: 7) + 1 ].
	self throws: [ ptr #putInt8: 0 - (self math: 2 pow: 7) - 1 ].
	k := 0 - (self math: 2 pow: 7).
	ptr #putInt8: k.
	k = ptr getInt8 ifFalse: [ ^'Failed #putInt8 ' ].

	self throws: [ ptr #putUInt8: 0 - 1 ].
	self throws: [ ptr #putUInt8: (self math: 2 pow: 8) ].
	k := (self math: 2 pow: 8) - 1.
	ptr #putUInt8: k.
	k = ptr getUInt8 ifFalse: [ ^'Failed #putUInt8 ' ].

	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test05

	| ptr k |
	ptr := self newPointer: 10.

	self throws: [ ptr #putInt16: (self math: 2 pow: 15) + 1 ].
	self throws: [ ptr #putInt16: 0 - (self math: 2 pow: 15) - 1 ].
	k := 0 - (self math: 2 pow: 15).
	ptr #putInt16: k.
	k = ptr getInt16 ifFalse: [ ^'Failed #putInt16 ' ].

	self throws: [ ptr #putUInt16: 0 - 1 ].
	self throws: [ ptr #putUInt16: (self math: 2 pow: 16) ].
	k := (self math: 2 pow: 16) - 1.
	ptr #putUInt16: k.
	k = ptr getUInt16 ifFalse: [ ^'Failed #putUInt16 ' ].

	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test06

	| ptr k |
	ptr := self newPointer: 10.

	self throws: [ ptr #putInt32: (self math: 2 pow: 31) + 1 ].
	self throws: [ ptr #putInt32: 0 - (self math: 2 pow: 31) - 1 ].
	k := 0 - (self math: 2 pow: 31).
	ptr #putInt32: k.
	k = ptr getInt32 ifFalse: [ ^'Failed #putInt32 ' ].

	self throws: [ ptr #putUInt32: 0 - 1 ].
	self throws: [ ptr #putUInt32: (self math: 2 pow: 32) ].
	k := (self math: 2 pow: 31) - 1.
	ptr #putUInt32: k.
	k = ptr getUInt32 ifFalse: [ ^'Failed #putUInt32 ' ].

	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test07

	| ptr k nptr pp |
	ptr := self newPointer: 32.

	k := 1.5.
	ptr #putFloat: k.
	k = ptr getFloat ifFalse: [ ^'Failed #putFloat ' ].

	k := 1000.005.
	ptr #putDouble: k.
	k = ptr getDouble ifFalse: [ ^'Failed #putDouble ' ].

	nptr := self newPointer: 32.
	k := 1234.5678.
	nptr #putDouble: k.
	ptr #putPointer: nptr. pp := ptr #getPointer.
	nptr#address = pp#address ifFalse: [ ^'Failed #putDouble-getPointer ' ].
	k = ptr getPointer getDouble ifFalse: [ ^'Failed #getDouble'].

	32 = nptr#allocated ifFalse: [ ^'Failed nptr allocated' ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test08

	| ptr |
	ptr := self newPointer: 1024.
	k := 'Hello World!!'.
	ptr #putCString: k.
	k = ptr getCString ifFalse: [ ^'Failed #putCString ' ].	
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test09

	| ptr |
	ptr := self newPointer: 70.
	ptr #putByte: 6. 6 = ptr getByte ifFalse: [ ^'Failed putByte 6' ].
	ptr #putChar: 6. 6 = ptr getChar ifFalse: [ ^'Failed putChar 6' ].
	ptr #putChar: 0 - 6. 0 - 6 = ptr getChar ifFalse: [ ^'Failed putChar -6' ].
	self throws: [ ptr putUChar: 0 - 8 ].
	ptr #putChar: 0 - 8. (8 ~= ptr getUChar and: [ ptr getUChar > 0 ]) ifFalse: [ ^'Failed putUChar -8' ].
	ptr #putChar: 8. 8 = ptr getChar ifFalse: [ ^'Failed putChar 8' ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test10

	| ptr |
	ptr := self newPointer: 70.
	ptr #putShort: 9. 9 = ptr getShort ifFalse: [ ^'Failed putShort 9' ].
	ptr #putShort: 0 - 9. 0 - 9 = ptr getShort ifFalse: [ ^'Failed putShort -9' ].
	self throws: [ ptr putUShort: 0 - 8 ].
	ptr #putShort: 0 - 8. (8 ~= ptr getUShort and: [ ptr getUShort > 0 ]) ifFalse: [ ^'Failed putUShort -8' ].
	ptr #putUShort: 11. 11 = ptr getUShort ifFalse: [ ^'Failed putUShort 11' ].
	ptr #putInt: 12. 12 = ptr getInt ifFalse: [ ^'Failed putInt 12' ].
	ptr #putInt: 0 - 12. 0 - 12 = ptr getInt ifFalse: [ ^'Failed putInt -12' ].
	self throws: [ ptr putUInt: 0 - 8 ].
	ptr #putInt: 0 - 8. (8 ~= ptr getUInt and: [ ptr getUInt > 0 ]) ifFalse: [ ^'Failed putUShort -8' ].
	ptr #putUInt: 13. 13 = ptr getUInt ifFalse: [ ^'Failed putUInt 13' ].
	self throws: [ ptr putULong: 0 - 8 ].
	ptr #putLong: 0 - 8. (8 ~= ptr getULong and: [ ptr getULong > 0 ]) ifFalse: [ ^'Failed putULong -8' ].
	ptr #putULong: 11. 11 = ptr getULong ifFalse: [ ^'Failed putULong 11' ].
	ptr #putULongLong: 16. 16 = ptr getULongLong ifFalse: [ ^'Failed putULongLong 16' ].
	ptr #putLongLong: 0 - 16. 0 - 16 = ptr getLongLong ifFalse: [ ^'Failed putULongLong -16' ].
	self throws: [ ptr putULongLong: 0 - 8 ].
	ptr #putLongLong: 0 - 8. (8 ~= ptr getULongLong and: [ ptr getULongLong > 0 ]) ifFalse: [ ^'Failed putULongLong -8' ].
	ptr #putULongLong: 17. 17 = ptr getULongLong ifFalse: [ ^'Failed putULongLong 17' ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test11
	| test pointer |
	test := [:what|
		pointer := self staticFunction: what.
		(self isFFIPointer: pointer) ifFalse: [ ^'Failed function pointer ',what ].
	 ].
	test value: #dlopen; value: #dlclose; value: #dlsym; value: #dlerror.
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test12
	| definition test |
	definition := self defineStructure: #(
		#( #byte #first )
		#( #byte #last )
	).
	test := self newStruct: definition.

	test#first := 50. 50 = test#first ifFalse: [ ^'Failed field #first ' ].
	test#last := 100. 100 = test#last ifFalse: [ ^'Failed field #last ' ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test13
	| definition test testPtr tuples success |
	definition := self defineStructure: #(
		#( #byte #byteVal )
		#( #int8 #int8Val )
		#( #int16 #int16Val )
		#( #uint16 #uint16Val )
		#( #int32 #int32Val )
		#( #uint32 #uint32Val )
		#( #float #floatVal )
		#( #double #doubleVal )
		#( #pointer #pointerVal )
	).
	testPtr := self newPointer: 4.
	tuples := #(
		#( #byteVal 100 )
		#( #int8Val 100 )
		#( #int16Val 1000 )
		#( #uint16Val 1000 )
		#( #int32Val 10000 )
		#( #uint32Val 10000 )
		#( #floatVal 1.25 )
		#( #doubleVal 10000.0005 )
		), (Array with: (Array with: #pointerVal with: testPtr)).
	tuples do: [:tuple| tuple first first = $i ifTrue: [ tuple at: 2 put: 0 - tuple last ] ].

	test := self newStruct: definition with: (self json: tuples).

	tuples do: [:tuple|
		success := tuple first = #pointerVal
			ifTrue: [ (test#pointerVal basicAt: #address) = (tuple last basicAt: #address) ]
			ifFalse: [ tuple last = test[tuple first] ].
		success ifFalse: [ ^'Failed field #',tuple first ]
	].

	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test14

	self throws: [ self defineStructure: #( #( #byte #a ) #( #byte #a ) ) ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test15
	| test pointer |
	test := [:what|
		pointer := self ffiType: what.
		(self isFFIPointer: pointer) ifFalse: [ ^'Failed FFI_TYPE pointer ',what ].
	 ].
	test value: #void; value: #int8.
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test16
	| tcif cifat address |
	tcif := self newCIF: #int32 types: #( #int32 ).
	(self isFFIPointer: tcif getArgTypesPointer) ifFalse: [ ^'Failed getArgTypesPointer' ].
	cifat := tcif getArgTypesPointer #seek: 0.
	address := (cifat #getPointer: true) basicAt: #address.
	((self ffiType: #int32) basicAt: #address) = address ifFalse: [ ^'Failed #int32 type' ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test17
	| ff abs |
	ff := self newCdecl: (self staticFunction: #abs) return: #int32 types: #( #int32 ).
	abs := ff getFunction.
	(# #abs: 0 - 1234) = 1234 ifFalse: [ ^'Failed abs' ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test18
	| ffi ptr derefer hello |
	ffi := self ffi.
	ptr := self pointerAlloc: #int32 value: 1234.
	ptr getInt32 = 1234 ifFalse: [ ^'Failed getInt32' ].
	(ffi #derefValuePtr: #int32 pointer: ptr) = 1234 ifFalse: [ ^'Failed derefValuePtr #int32' ].
	derefer := ffi #derefValuePtrFunc: #int32.
	1234 = (# #derefer: ptr) ifFalse: [ ^'Failed #int32 derefer' ].

	hello := 'Hello Smalltalk!!'.
	ptr := self pointerAlloc: #string value: hello.
	(ffi #derefValuePtr: #string pointer: ptr) = hello ifFalse: [ ^'Failed derefValuePtr #string' ].
	derefer := ffi #derefValuePtrFunc: #string.
	hello = (# #derefer: ptr) ifFalse: [ ^'Failed #string derefer' ].

	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test19
	| api |
	api := self cdecl: (self staticFunction: #abs) return: #int32 types: #( #int32 ).
	1234 = (# #api: -1234) ifFalse: [ ^'Failed abs' ].

	api := self cdecl: (self staticFunction: #atoi) return: #int32 types: #( #string ).
	1234 = (# #api: '1234') ifFalse: [ ^'Failed atoi' ].

	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test20
	^#Ok ,' - Running on ', NodeJS os platform! !

! FFIBasicTests class methodsFor: 'tests' !
test21
	| extensions |
	extensions := self ffi basicAt: 'PLATFORM_LIBRARY_EXTENSIONS'.
	^#Ok ,' extensions=',(extensions basicAt: NodeJS os platform)! !

! FFIBasicTests class methodsFor: 'tests-api' !
test22
	| api |
	api := self kernelLibrary #get: #GetCommandLineA.
	api := self stdCall: api return: #string types: #().
	^#Ok ,' - api: GetCommandLineA =', # #api! !

! FFIBasicTests class methodsFor: 'tests-api' !
test23
	| api |
	api := self kernelLibrary #get: #GetCurrentThreadId.
	api := self stdCall: api return: #uint32 types: #().
	^#Ok ,' - api: GetCurrentThreadId =', # #api! !

! FFIBasicTests class methodsFor: 'tests' !
test24
	| api spec fd |
true ifTrue: [ ^#Failed ,' - ToDo: calls to "nil" module (exported by running VM).' ].
	spec := #(
		#( #fopen #( #pointer #( #string #string ) ) )
		#( #fclose #( #int32 #( #pointer ) ) )
	).
	api := self newLibrary: nil functions: (self json: spec).
	fd := api #fopen: Smalltalk current imageFileName.
	fd isNil ifTrue: [ ^'Failed fopen ', Smalltalk current imageFileName ].
	fd #isNull ifTrue: [ ^'Failed fopen return #isNull' ].
	(api #fclose: fd) = 0 ifFalse: [ ^'Failed #fclose' ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test25
	| count cifPtr clz api |
	count := 0.
	cifPtr := self newCIF: #int32 types: #( #int32 ).
	clz := self
		new: #CallbackInfo
		with: cifPtr getPointer
		with: [:result :args | count := count + 1 ].
	api := self cdecl: clz#pointer return: #int32 types: #( #int32 ).
	# #api: 1. 1 = count ifFalse: [ ^'Failed #1' ].
	# #api: 2. 2 = count ifFalse: [ ^'Failed #2' ].
	count = 0 ifTrue: [ ^'Failed - No callbacks' ].
	^#Ok ,' - count=',count asString! !

! FFIBasicTests class methodsFor: 'tests' !
test26
	| callback ptr |
	callback := self new: #Callback with: #( #int32 #( #int32 ) ) with: [:value| 0 - value ].
	ptr := callback getPointer.
	api := self cdecl: ptr return: #int32 types: #( #int32 ).
	1234 = (# #api: -1234) ifFalse: [ ^'Failed - wrong result' ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test27
	| executed success |
	executed := success := false.
	api := self cdecl: (self staticFunction: #abs) return: #int32 types: #( #int32 ) async: true.
	(# #api: 0 - 1234) #on: #success do: [:result|
		executed := true.
		success := 1234 = result.
	].
	[ executed ifTrue: [ #{console} #log: 'test27 step (waiting async execution)' ] ] valueWithTimeout: 250. "allow the event loop to complete"
	executed ifFalse: [ ^#Failed ,' - to execute async function.' ].
	success ifFalse: [ ^#Failed ,' - async call, wrong result.' ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test28
	| p b hw |
	p := self newPointer: 128.
	b := p toBuffer.
	p#allocated = b#length ifFalse: [ ^'Failed toBuffer' ].
	b #write: #hello asAsciiZ.
	p getCString = #hello ifFalse: [ ^'Failed hello result is ',p getCString ].
	hw := 'hello world'.
	p #putCString: hw.
	hw = (b #slice: 0 length: hw size) toString ifFalse: [ ^'Fail slice world' ].
	b[2] := #L #charCodeAt: 0.
	p getCString = 'heLlo world' ifFalse: [ ^'Failed heLlo' ].
	^#Ok! !

! FFIBasicTests class methodsFor: 'tests' !
test29
	| tuples definition test api |
	tuples := #(
		#( #uint16 #processorArchitecture )
		#( #uint16 #reserved )
		#( #ulong #pageSize )
		#( #ulong #minimumApplicationAddress )
		#( #ulong #maximumApplicationAddress )
		#( #ulong #activeProcessorMask )
		#( #ulong #numberOfProcessors )
		#( #ulong #processorType )
		#( #ulong #allocationGranularity )
		#( #uint16 #processorLevel )
		#( #uint16 #processorRevision )
	).
	definition := self defineStructure: tuples.
	test := self newStruct: definition.

	definition['__structInfo__']#size ~= 36 ifTrue: [
		^'Failed structure size is ',definition['__structInfo__']#size asString
	].

	api := self kernelLibrary #get: #GetNativeSystemInfo.
	api := self stdCall: api return: #void types: #( #pointer ).
	# #api: test['__pointer'].
	^#Ok ,(tuples inject: ' - api: GetSystemInfo' into: [:total :tuple|
		total ,'
		',tuple last ,'=' ,(test basicAt: tuple last) asString
	])! !

! FFIBasicTests class methodsFor: 'tests' !
test30
	| api hProcess pBoolean |
	pBoolean := self newPointer: 4.
	api := self kernelLibrary #get: #GetCurrentProcess.
	api := self stdCall: api return: #int32 types: #().
	hProcess := # #api.
	api := self kernelLibrary #get: #IsWow64Process.
	api := self stdCall: api return: #uint32 types: #( #int32 #pointer ).
	(# #api: hProcess with: pBoolean) = 0 ifTrue: [ ^'Function call failed' ].
	^#Ok ,' - api: isWow64Process = ',(pBoolean getULong ~= 0)! !

! FFIBasicTests class methodsFor: 'tests' !
test31
	| api |
	api := self kernelLibrary #get: #Sleep.
	api := self stdCall: api return: #void types: #( #uint32 ).
	# #api: 1000.
	^#Ok ,' - api: Sleep '! !

! FFIBasicTests class methodsFor: 'tests' !
test32
	| api result |
	api := self kernelLibrary #get: #Beep.
	api := self stdCall: api return: #uint32 types: #( #uint32 #uint32 ).
	result := # #api: 777 for: 500.
	^#Ok ,' - api: Beep ', result asString! !

! FFIBasicTests class methodsFor: 'tests' !
test33
	| api result p b size |
	size := 1024.
	p := self newPointer: size.
	api := self kernelLibrary #get: #GetCurrentDirectoryA.
	api := self stdCall: api return: #uint32 types: #( #uint32 #pointer ).
	result := # #api: size on: p.
	b := p toBuffer.
	^#Ok ,' - api: GetCurrentDirectory =', (b #slice: 0 length: size)! !

! FFIBasicTests class methodsFor: 'tests' !
test34
	| api result driveType |
	api := self kernelLibrary #get: #GetDriveTypeA.
	api := self stdCall: api return: #uint32 types: #( #string ).
	driveType := [:int|
		#(	#unknown #noRootDir #removable
			#fixed #remote #cdrom #ramdisk
		) at: int + 1 ifAbsent: [ 'invalid/undocumented API result']
	].
	^(Array with: #{null}) , #( 'c:\' 'd:\' 'e:\' 'f:\' 'g:\' )
	inject: #Ok ,' - api: GetDriveType' into: [:total :each|
		result := # #api: each.
		total ,'
		',each,' - ',(driveType value: result)
	]! !

! FFIBasicTests class methodsFor: 'tests' !
test35
	| api apiNames |
	apiNames := #(
		#GetLastError #GetTickCount #GetVersion
	).
	^apiNames inject: #Ok ,' - api: [return #uint32 no arguments]' into: [:total :each|
		api := self kernelLibrary #get: each.
		api := self stdCall: api return: #uint32 types: #().
		total,'
		',each,'=', # #api
	]! !

! FFIBasicTests class methodsFor: 'tests' !
test36
	| fields definition test api |
	fields := #( #year #month #dayOfWeek #day #hour #minute #second #milliseconds ).
	definition := self defineStructure: (fields collect: [:each| Array with: #uint16 with: each ]).
	test := self newStruct: definition.
	api := self kernelLibrary #get: #GetLocalTime.
	api := self stdCall: api return: #void types: #( #pointer ).
	# #api: test['__pointer'].
	^#Ok ,(fields inject: ' - api: GetLocalTime' into: [:total :each|
		total ,'
		',each ,'=' ,(test basicAt: each) asString
	])! !

! FFIBasicTests class methodsFor: 'tests' !
test37
	| fields definition test api |
	fields := #( #year #month #dayOfWeek #day #hour #minute #second #milliseconds ).
	definition := self defineStructure: (fields collect: [:each| Array with: #uint16 with: each ]).
	test := self newStruct: definition.
	api := self kernelLibrary #get: #GetSystemTime.
	api := self stdCall: api return: #void types: #( #pointer ).
	# #api: test['__pointer'].
	^#Ok ,(fields inject: ' - api: GetSystemTime' into: [:total :each|
		total ,'
		',each ,'=' ,(test basicAt: each) asString
	])! !

! FFIBasicTests class methodsFor: 'tests' !
test38
	| fields definition test api |
	fields := #(
		#( #uint32 #structLength )
		#( #uint32 #memoryLoad )
		#( #ulonglong #totalPhys )
		#( #ulonglong #availPhys )
		#( #ulonglong #totalPageFile )
		#( #ulonglong #availPageFile )
		#( #ulonglong #totalVirtual )
		#( #ulonglong #availVirtual )
		#( #ulonglong #availExtendedVirtual )
	).
	definition := self defineStructure: fields.
	test := self newStruct: definition.
	test#structLength := definition['__structInfo__']#size.
	api := self kernelLibrary #get: #GlobalMemoryStatusEx.
	api := self stdCall: api return: #int32 types: #( #pointer ).
	# #api: test['__pointer'].
	^#Ok ,(fields inject: ' - api: GlobalMemoryStatusEx' into: [:total :tuple|
		total ,'
		',tuple last ,'=' ,(test basicAt: tuple last) asString
	])! !

! FFIBasicTests class methodsFor: 'tests' !
test39
	| api |
	api := self kernelLibrary #get: #OutputDebugStringA.
	api := self stdCall: api return: #void types: #( #string ).
	# #api: self class name,'>>#test39 - This string should be output onto debug device.' asAsciiZ.
	^#Ok ,' - #OutputDebugString'! !

! FFIBasicTests class methodsFor: 'tests' !
test40
	| api result command |
	command := 'Notepad.exe'.
	api := self kernelLibrary #get: #WinExec.
	api := self stdCall: api return: #uint32 types: #( #string #uint32 ).
	result := # #api: command cmdShow: 0.
	result = 0 ifTrue: [ ^'Out of system resources' ].
	result = 11 ifTrue: [ ^'ERROR_BAD_FORMAT' ].
	result = 2 ifTrue: [ ^'ERROR_FILE_NOT_FOUND' ].
	result = 3 ifTrue: [ ^'ERROR_PATH_NOT_FOUND' ].
	result <= 31 ifTrue: [ ^'ERROR [',result asString,'] executing ',command ].
	^#Ok ,' - #WinExec ' ,command ,' result=',result asString! !

! FFIBasicTests class methodsFor: 'tests' !
test41
	| api heap result count entry fields definition success |
	api := self kernelLibrary #get: #GetProcessHeap.
	api := self stdCall: api return: #uint32 types: #().
	heap := # #api.
	heap = 0 ifTrue: [ ^'Failed to obtain process Heap' ].
	count := 0.
	api := self kernelLibrary #get: #HeapWalk.
	api := self stdCall: api return: #uint32 types: #( #uint32 #pointer ).

	fields := #(
		#( #pointer #lpData )
		#( #uint32 #count )
		#( #byte #overhead )
		#( #byte #regionIndex )
		#( #uint16 #flags )
		#( #uint32 #value1 )
		#( #uint32 #value2 )
		#( #uint32 #value3 )
		#( #uint32 #value4 )
	).
	definition := self defineStructure: fields.
	entry := self newStruct: definition.
	entry#lpData := #{null}.

	result := #Ok ,' - api: #HeapWalk [',heap asString,']'.
	[	success := # #api: heap with: entry['__pointer'].
		success ~= 0
	] whileTrue: [
		count := count + 1.
		self verbose ifTrue: [
			result := result,'
			#',count asString
				,' [',entry#count asString,' bytes] at 0x'
				,(entry#lpData#address #toString: 16) asUpperCase.
		].
	].
	^result ,'
	#HeapWalk complete. Steps=',count asString! !




! FFIBasicTests class methodsFor: 'tests' !
test99
	| api result |
	api := self kernelLibrary #get: #ExitProcess.
	api := self stdCall: api return: #void types: #( #uint32 ).
	false ifTrue: [ result := # #api: 7. ]. "we do not want to kill this process during testing"
	^#Ok ,' - api: ExitProcess - We survived.'! !

