Object
 subclass: #FFIObject
 instanceVariableNames: ''
 classVariableNames: ' FFIKernelLibrary '
 poolDictionaries: ''
 category: 'FFICore'!
FFIObject comment: '
	Model objects of FFI implementation (running on node.js execution engine).
'!

! FFIObject class methodsFor: 'utilities-json' !
json: tuples
	" Private - Return a native object with tuples. "

	| result |
	result := {' {} '}.
	tuples do: [:tuple| result[tuple first] := tuple last ].
	^result! !

! FFIObject class methodsFor: 'utilities-GC' !
gc: aBoolean
	" V8 - Garbage collect.
	There is a GC extension which can be enabled through the flag --expose-gc .
	With this extension enabled calling gc() from JavaScript performs a full GC.
	"

{' if (typeof gc == "function") gc(aBoolean); '}! !

! FFIObject class methodsFor: 'utilities-GC' !
gc
	" V8 - Full Garbage collect.
	There is a GC extension which can be enabled through the flag --expose-gc .
	With this extension enabled calling gc() from JavaScript performs a full GC.
	"

	self gc: true! !

! FFIObject class methodsFor: 'constants' !
defaultABI
	" Return the constant value for this type of ABI calls. "

	^0! !

! FFIObject class methodsFor: 'constants' !
cdeclABI
	" Return the constant value for this type of ABI calls. "

	^1! !

! FFIObject class methodsFor: 'constants' !
stdCallABI
	" Return the constant value for this type of ABI calls. "

	^2! !

! FFIObject class methodsFor: 'utilities' !
ffi
	" Private - Return the FFI module. "

	^# #require: 'ffi'! !

! FFIObject class methodsFor: 'utilities' !
bindings
	" Private - Return the bindings. "

	^self ffi basicAt: #Bindings! !

! FFIObject class methodsFor: 'utilities' !
newPointer: sizeInBytes
	" Private - Return a Pointer to external memory of size sizeInBytes. "

	| ffi |
	ffi := self ffi.
{'	return new ffi.Pointer(sizeInBytes); '}! !

! FFIObject class methodsFor: 'utilities' !
defineStructure: tuples
	" Private - Return a new structure definition. "

	| ffi |
	ffi := self ffi.
{'	return new ffi.Struct(tuples); '}! !

! FFIObject class methodsFor: 'utilities' !
newStruct: structureDefinition
	" Private - Return a new record with structure definition. "

{'	return new structureDefinition();'}! !

! FFIObject class methodsFor: 'utilities' !
newStruct: structureDefinition with: initializer
	" Private - Return a new record with structure definition. "

{'	return new structureDefinition(initializer);'}! !

! FFIObject class methodsFor: 'utilities' !
staticFunction: key
	" Private - Return the static function at key. "

	^self staticFunctions basicAt: key! !

! FFIObject class methodsFor: 'utilities' !
staticFunctions
	" Private - Return the static functions object. "

	^self bindings basicAt: #StaticFunctions! !

! FFIObject class methodsFor: 'utilities' !
ffiType: key
	" Private - Return the FFI_Type at key. "

	^self ffiTypes basicAt: key ! !


! FFIObject class methodsFor: 'utilities' !
ffiTypes
	" Private - Return the FFI_Types. "

	^self bindings basicAt: 'FFI_TYPES'! !

! FFIObject class methodsFor: 'utilities' !
isFFIPointer: aJSObject
	" Private - Return true if aJSObject is instance of Pointer. "

	| pointer |
	aJSObject isNil ifTrue: [ ^false ].
	pointer := self ffi basicAt: #Pointer.
{'	return (aJSObject instanceof pointer); '}! !

! FFIObject class methodsFor: 'utilities' !
pointerAlloc: type value: value
	" Private - Return a pointer to externally allocated memory of type initialized with value. "

	^(self ffi basicAt: #Pointer) #alloc: type value: value! !

! FFIObject class methodsFor: 'utilities' !
math: x pow: y
	" Private - Return the value of x to the power of y. "

{'	return Math.pow(x,y); '}! !

! FFIObject class methodsFor: 'utilities' !
new: aFFIClass with: first
	" Private - Return aFFIClass instance with arguments. "

	| ffi |
	ffi := self ffi.
{'	return new ffi[aFFIClass](first); '}! !

! FFIObject class methodsFor: 'utilities' !
new: aFFIClass with: first with: second
	" Private - Return aFFIClass instance with arguments. "

	| ffi |
	ffi := self ffi.
{'	return new ffi[aFFIClass](first,second); '}! !

! FFIObject class methodsFor: 'utilities' !
new: aFFIClass with: first with: second with: third
	" Private - Return aFFIClass instance with arguments. "

	| ffi |
	ffi := self ffi.
{'	return new ffi[aFFIClass](first,second,third); '}! !

! FFIObject class methodsFor: 'utilities' !
new: aFFIClass with: first with: second with: third with: fourth
	" Private - Return aFFIClass instance with arguments. "

	| ffi |
	ffi := self ffi.
{'	return new ffi[aFFIClass](first,second,third,fourth); '}! !

! FFIObject class methodsFor: 'utilities' !
new: aFFIClass with: first with: second with: third with: fourth with: fifth
	" Private - Return aFFIClass instance with arguments. "

	| ffi |
	ffi := self ffi.
{'	return new ffi[aFFIClass](first,second,third,fourth,fifth); '}! !

! FFIObject class methodsFor: 'utilities' !
newCIF: returnType types: argumentTypes abi: abi
	" Private - Return a Call Interface. "

	^self new: #CIF with: returnType with: argumentTypes with: abi! !

! FFIObject class methodsFor: 'utilities' !
newCIF: returnType types: argumentTypes
	" Private - Return a Call Interface. "

	^self newCIF: returnType types: argumentTypes abi: self defaultABI! !

! FFIObject class methodsFor: 'utilities' !
newForeignFunction: aFunctionPtr abi: abi return: returnType types: argumentTypes async: async
	" Private - Return a foreign function instance. "

	aFunctionPtr isNil ifTrue: [ ^self error: 'Invalid function pointer' ].
	^self
		new: #ForeignFunction
		with: aFunctionPtr with: returnType
		with: argumentTypes with: async with: abi! !

! FFIObject class methodsFor: 'utilities' !
newForeignFunction: aFunctionPtr abi: abi return: returnType types: argumentTypes
	" Private - Return a foreign function instance. "

	^self
		newForeignFunction: aFunctionPtr
		abi: abi return: returnType
		types: argumentTypes async: false! !

! FFIObject class methodsFor: 'utilities' !
newCdecl: aFunctionPtr return: returnType types: argumentTypes
	" Private - Return a foreign function instance. "

	^self	newForeignFunction: aFunctionPtr
		abi: self cdeclABI return: returnType
		types: argumentTypes! !

! FFIObject class methodsFor: 'utilities' !
newSTDCall: aFunctionPtr return: returnType types: argumentTypes
	" Private - Return a foreign function instance. "

	^self	newForeignFunction: aFunctionPtr
		abi: self stdCallABI return: returnType
		types: argumentTypes! !

! FFIObject class methodsFor: 'utilities' !
foreignFunction: aFunctionPtr abi: abi return: returnType types: argumentTypes
	" Private - Return a foreign function object. "

	| ffi |
	ffi := self ffi.
	aFunctionPtr isNil ifTrue: [ ^self error: 'Invalid function pointer' ].
{'	return ffi.ForeignFunction.build(aFunctionPtr,returnType,argumentTypes,false,abi); '}! !

! FFIObject class methodsFor: 'utilities' !
cdecl: aFunctionPtr return: returnType types: argumentTypes
	" Private - Return a foreign function object. "

	^self
		foreignFunction: aFunctionPtr
		abi: self cdeclABI return: returnType
		types: argumentTypes! !

! FFIObject class methodsFor: 'utilities' !
stdCall: aFunctionPtr return: returnType types: argumentTypes
	" Private - Return a foreign function object. "

	^self
		foreignFunction: aFunctionPtr
		abi: self stdCallABI return: returnType
		types: argumentTypes! !

! FFIObject class methodsFor: 'utilities' !
foreignFunction: aFunctionPtr abi: abi return: returnType types: argumentTypes async: async
	" Private - Return a foreign function object. "

	| ffi |
	ffi := self ffi.
	aFunctionPtr isNil ifTrue: [ ^self error: 'Invalid function pointer' ].
{'	return ffi.ForeignFunction.build(aFunctionPtr,returnType,argumentTypes,async,abi); '}! !

! FFIObject class methodsFor: 'utilities' !
cdecl: aFunctionPtr return: returnType types: argumentTypes async: async
	" Private - Return a foreign function object. "

	^self
		foreignFunction: aFunctionPtr
		abi: self cdeclABI return: returnType
		types: argumentTypes async: async! !

! FFIObject class methodsFor: 'utilities' !
stdCall: aFunctionPtr return: returnType types: argumentTypes async: async
	" Private - Return a foreign function object. "

	^self
		foreignFunction: aFunctionPtr
		abi: self stdCallABI return: returnType
		types: argumentTypes async: async! !

! FFIObject class methodsFor: 'private' !
libName: libraryName
	" Private - Return a valid libraryName. "

	^libraryName isNil ifTrue: [ #{null} ] ifFalse: [ libraryName ]! !

! FFIObject class methodsFor: 'utilities' !
newDynamicLibrary: libraryName
	" Private - Return a DynamicLibrary instance. "

	^self new: #DynamicLibrary with: (self libName: libraryName)! !

! FFIObject class methodsFor: 'utilities-win32' !
kernelLibrary
	" Private - Return The kernel library. "

	FFIKernelLibrary isNil ifTrue: [ FFIKernelLibrary := self newDynamicLibrary: 'Kernel32' ].
	^FFIKernelLibrary! !

! FFIObject class methodsFor: 'utilities' !
newLibrary: libraryName functions: initializer
	" Private - Return a new library. "

	^self new: #Library with: (self libName: libraryName) with: initializer! !
