FFIObject
 subclass: #FFITests
 instanceVariableNames: ''
 classVariableNames: ' '
 poolDictionaries: ''
 category: 'FFITests'!
FFITests comment: '
	Abstract testing of FFI framework for S8.
'!

! FFITests class methodsFor: 'settings' !
verbose
	" Private - Return true if verbose output is enabled. "

	^false! !

! FFITests class methodsFor: 'query' !
contents
	" Return a string with the results of running all samples in the receiver. "

	^self allSamples inject: String new into: [:total :assoc|
		(self verbose not and: [assoc value asString includes: #Ok]) ifTrue: [ total ] ifFalse: [
		total , assoc key asString, ' ', assoc value asString,'
']
	]! !

! FFITests class methodsFor: 'samples' !
allSamples
	" Return all samples in the receiver and its subclasses. "

	^self withAllSubclasses reversed inject: #() into: [:total :each|
		total , each samples , each new samples
	]! !

! FFITests class methodsFor: 'samples' !
samples
	" Return the result of running all the samples in the receiver. "

	^self samplesOn: self! !

! FFITests methodsFor: 'samples' !
samples
	" Return the result of running all the samples in the receiver. "

	^self class samplesOn: self! !

! FFITests class methodsFor: 'samples' !
samplesOn: anObject
	" Private - Return the result of running all the samples in an object.
	The  result will be an association with
		key = sample selector
		value = the result of executing the sample selector on anObject.
	"

	^(self sampleSelectorsFor: anObject) collect: [:selector|

self print: anObject asString,'>>#',selector.

		Association
			key: anObject asString, '>>#', selector
			value: ([self gc. anObject perform: selector] on: Error
				do: [:ex| ex class name ,': ' ,ex description,'
'					,ex stack
				])
	]! !

! FFITests class methodsFor: 'samples' !
sampleSelectorsFor: anObject
	" Private - Return the sample selectors in an object. "

	^(self selectorsOf: anObject startingWith: #example)
	,(self selectorsOf: anObject startingWith: #test)! !

! FFITests class methodsFor: 'samples' !
selectorsOf: anObject startingWith: text
	" Return the selectors implemented by anObject's class matching text. "

	^(anObject class selectors
		select: [:each| (each indexOf: text) = 1 ]
		) sorted! !

! FFITests class methodsFor: 'utilities' !
vprint: what
	" Private - Print what if verbose output is enabled. "

	^self verbose ifTrue: [ self print: what ]! !

! FFITests class methodsFor: 'utilities' !
keysOf: anObject
	" Private - Return the property keys of anObject. "

{'	return Object.keys(anObject); '}! !

! FFITests class methodsFor: 'utilities' !
inspect: keys of: anObject as: title
	" Private - Return inspect information about anObject. "

	| result |
result := 'inspecting ',title.
self vprint: result.
	result := keys inject: result into: [:total :each|
self vprint: '	',each,'=',anObject[each].
		total,'
	',each,'=',anObject[each]
	].
self vprint: '------------',title,' ',anObject.
	^result! !

! FFITests class methodsFor: 'utilities' !
inspect: anObject as: title
	" Private - Return inspect information about anObject. "

	^self inspect: (self keysOf: anObject) of: anObject as: title! !

! FFITests class methodsFor: 'utilities' !
inspect: anObject
	" Private - Return inspect information about anObject. "

	^self inspect: anObject as: ''! !

! FFITests class methodsFor: 'utilities' !
throws: aBlock
	" Private - Assert execution of aBlock throw an exception. "

	^self throw: Error evaluating: aBlock! !

! FFITests class methodsFor: 'utilities' !
throw: exception evaluating: aBlock
	" Private - Assert execution of aBlock throw an exception. "

	aBlock on: exception do: [^'Ok'].
	^self error: 'Must throw ',exception asString! !
