FFITests
 subclass: #TestDLL
 instanceVariableNames: ''
 classVariableNames: ' '
 poolDictionaries: ''
 category: 'TestDLL'!
TestDLL comment: '
	Tests for API calls defined in TestDLL.
	Note: Edit the #libraryFileName implementation to point to test DLL file.
'!
TestDLL class instanceVariableNames: ' library '!

! TestDLL class methodsFor: 'accessing' !
library
	" Private - Return the library of the receiver. "

	library isNil ifTrue: [ library := self newDynamicLibrary: self libraryFileName ].
	^library! !

! TestDLL class methodsFor: 'accessing' !
libraryFileName
	" Private - Return the library fileName. "

	^'testDLL\TestDLLd.dll'! !

! TestDLL class methodsFor: 'tests' !
test00
	" Inform all is ok, library loaded and ready to run tests. "

	self library isNil ifTrue: [ ^'Failed to load library ',self libraryFileName ].
	self library close. library := nil.
	self library isNil ifTrue: [ ^'Failed to reload library ',self libraryFileName ].
	^#Ok ,' - Running ',self name! !

! TestDLL class methodsFor: 'tests-api' !
test02
	| functions api mangled |
	functions := #(
		#( #void 42 )
		#( #Min -2147483648 )
		#( #Max 2147483647 )
		#( 0 0 ) #( 1 1 )
		#( #1Negated -1)
	).
	functions do: [:tuple|
		mangled := '_intSTD',tuple first,'@0'.
		api := self library #get: mangled.
		api := self stdCall: api return: #int32 types: #().
		tuple last = # #api ifFalse: [ ^'Failed ',mangled ]
	].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test03
	| functions api mangled |
	functions := #(
		#( #void 42 )
		#( #Min -2147483648 )
		#( #Max 2147483647 )
		#( 0 0 ) #( 1 1 )
		#( #1Negated -1)
	).
	functions do: [:tuple|
		mangled := #intC ,tuple first.
		api := self library #get: mangled.
		api := self cdecl: api return: #int32 types: #().
		tuple last = # #api ifFalse: [ ^'Failed ',mangled ]
	].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test04
	| api mangled k |
	k := 123.
	mangled := '_intSTDintNegated@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #int32 ).
	0 - k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test05
	| api mangled k |
	k := 123.
	mangled := #intCintNegated.
	api := self library #get: mangled.
	api := self cdecl: api return: #int32 types: #( #int32 ).
	0 - k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test06
	| api mangled |
true ifTrue: [ ^#Ok ,' - FFI do not support type #boolean' ].
	mangled := '_intSTDboolean@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #int32 ).
	(# #api: false) = 0 ifFalse: [
		^'Failed - ' ,mangled
			,' true=',((# #api: true) #toString: 16) asUppercase 
			,' false=',((# #api: false) #toString: 16) asUppercase
	].
	^#Ok ! !

! TestDLL class methodsFor: 'tests-api' !
test07
	| api mangled |
true ifTrue: [ ^#Ok ,' - FFI do not support type #boolean' ].
	mangled := #intCboolean.
	api := self library #get: mangled.
	api := self cdecl: api return: #int32 types: #( #int32 ).
	(# #api: false) = 0 ifFalse: [
		^'Failed - ' ,mangled
			,' true=',((# #api: true) #toString: 16) asUppercase 
			,' false=',((# #api: false) #toString: 16) asUppercase
	].
	^#Ok ! !

! TestDLL class methodsFor: 'tests-api' !
test08
	| api mangled k |
	k := 1000.005.
	mangled := '_intSTDdouble@8'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #double ).
	1000 = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test09
	| api mangled k |
	k := 1000.005.
	mangled := #intCdouble.
	api := self library #get: mangled.
	api := self cdecl: api return: #int32 types: #( #double ).
	1000 = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test10
	| api mangled k pResult pObject |
	k := 'Hello STD Smalltalk'.
	pObject := self newPointer: 8. pObject #putObject: k.
	mangled := '_ptrSTDself@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #pointer types: #( #pointer ).
	pResult := # #api: pObject.
	k = pResult getObject ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test11
	| api mangled k pResult pObject |
	k := 'Hello C Smalltalk'.
	pObject := self newPointer: 8. pObject #putObject: k.
	mangled := #ptrCself.
	api := self library #get: mangled.
	api := self cdecl: api return: #pointer types: #( #pointer ).
	pResult := # #api: pObject.
	k = pResult getObject ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test12
	| api mangled k |
	k := 100.
	mangled := '_intSTDshortNegated@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #short ).
	0 - k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test13
	| api mangled k |
	k := 100.
	mangled := #intCshortNegated.
	api := self library #get: mangled.
	api := self cdecl: api return: #int32 types: #( #short ).
	0 - k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test14
	| api mangled k |
	k := 100.
	mangled := '_intSTDshort@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #short ).
	k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test15
	| api mangled k |
	k := 100.
	mangled := #intCshort.
	api := self library #get: mangled.
	api := self cdecl: api return: #int32 types: #( #short ).
	k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test16
	| api mangled k |
	k := 100.
	mangled := '_ulongSTDshort@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #ulong types: #( #short ).
	k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test17
	| api mangled k |
	k := 100.
	mangled := #ulongCshort.
	api := self library #get: mangled.
	api := self cdecl: api return: #ulong types: #( #short ).
	k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test18
	| api mangled k |
	k := 1012340.
	mangled := '_ulongSTDulong@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #ulong types: #( #ulong ).
	k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test19
	| api mangled k |
	k := 1043210.
	mangled := #ulongCulong.
	api := self library #get: mangled.
	api := self cdecl: api return: #ulong types: #( #ulong ).
	k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test20
	| api mangled k |
	k := -12340.
	mangled := '_doubleSTDint@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #double types: #( #int32 ).
	k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test20f
	| api mangled k |
	k := -12340.
	mangled := '_floatSTDint@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #float types: #( #int32 ).
	k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test21
	| api mangled pointer cookie |
	mangled := '_cppSTDvoid@0'.
	api := self library #get: mangled.
	api := self stdCall: api return: #pointer types: #().
	pointer := # api.
	cookie := 1.
	mangled := '_intSTDcpp@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #pointer ).
	cookie = (# #api: pointer) ifFalse: [ ^'Failed ',mangled ,' cookie failure' ].
	mangled := '_releaseSTDcpp@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #void types: #( #pointer ).
	# #api: pointer.
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test22
	| functions api mangled |
	functions := #(
		#( #Min -32768 )
		#( #Max 32767 )
		#( 0 0 ) #( 1 1 )
		#( #1Negated -1)
	).
	functions do: [:tuple|
		mangled := '_shortSTD' ,tuple first ,'@0'.
		api := self library #get: mangled.
		api := self stdCall: api return: #short types: #().
true ifTrue: [ ^#Failed ,' - ToDo: Heap corruption calling functions returning #short ' ].
		tuple last = # #api ifFalse: [ ^'Failed ',mangled,' = ',# #api ]
	].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test23
	| api mangled k |
	k := 100.
	mangled := '_intSTDint1@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #int32 ).
	k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test24
	| api mangled k |
	k := 100.
	mangled := #intCint1.
	api := self library #get: mangled.
	api := self cdecl: api return: #int32 types: #( #int32 ).
	k = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test25
	| api mangled |
	mangled := '_intSTDint2@8'.
	api := self library #get: mangled.
	api := self
		stdCall: api return: #int32
		types: #( #int32 #int32 ).
	2 = (# #api: 1 with: 1) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test26
	| api mangled |
	mangled := #intCint2.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #int32
		types: #( #int32 #int32 ).
	2 = (# #api: 1 with: 1) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test27
	| api mangled |
	mangled := '_intSTDint3@12'.
	api := self library #get: mangled.
	api := self
		stdCall: api return: #int32
		types: #( #int32 #int32 #int32 ).
	3 = (# #api: 1 with: 1 with: 1) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test28
	| api mangled |
	mangled := #intCint3.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #int32
		types: #( #int32 #int32 #int32 ).
	3 = (# #api: 1 with: 1 with: 1) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test29
	| api mangled |
	mangled := '_intSTDint4@16'.
	api := self library #get: mangled.
	api := self
		stdCall: api return: #int32
		types: #( #int32 #int32 #int32 #int32 ).
	4 = (# #api: 1 with: 1 with: 1 with: 1) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test30
	| api mangled |
	mangled := #intCint4.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #int32
		types: #( #int32 #int32 #int32 #int32 ).
	4 = (# #api: 1 with: 1 with: 1 with: 1) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test31
	| api mangled |
	mangled := '_intSTDint5@20'.
	api := self library #get: mangled.
	api := self
		stdCall: api return: #int32
		types: #( #int32 #int32 #int32 #int32 #int32 ).
	5 = (# #api: 1 with: 1 with: 1 with: 1 with: 1) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test32
	| api mangled |
	mangled := #intCint5.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #int32
		types: #( #int32 #int32 #int32 #int32 #int32 ).
	5 = (# #api: 1 with: 1 with: 1 with: 1 with: 1) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test33
	| api mangled |
	mangled := '_intSTDint10@40'.
	api := self library #get: mangled.
	api := self
		stdCall: api return: #int32
		types: #(
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
		).
	10 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test34
	| api mangled |
	mangled := #intCint10.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #int32
		types: #(
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
		).
	10 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test35
	| api mangled |
	mangled := '_intSTDint20@80'.
	api := self library #get: mangled.
	api := self
		stdCall: api return: #int32
		types: #(
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
		).
	20 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test36
	| api mangled |
	mangled := #intCint20.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #int32
		types: #(
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
		).
	20 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test37
	| api mangled |
	mangled := '_intSTDint40@160'.
	api := self library #get: mangled.
	api := self
		stdCall: api return: #int32
		types: #(
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
		).
	40 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test36
	| api mangled |
	mangled := #intCint40.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #int32
		types: #(
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
		).
	40 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test37
	| api mangled |
	mangled := '_intSTDint80@320'.
	api := self library #get: mangled.
	api := self
		stdCall: api return: #int32
		types: #(
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
		).
	80 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test38
	| api mangled |
	mangled := #intCint80.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #int32
		types: #(
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
			#int32 #int32 #int32 #int32 #int32
		).
	80 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test39
	| api mangled |
	mangled := #doubleCdouble40.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #double
		types: #(
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
		).
	40 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test40
	| api mangled |
	mangled := #doubleCdouble80.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #double
		types: #(
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
			#double #double #double #double #double
		).
	80 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test41
	| api mangled k |
	k := 100.2.
	mangled := '_intSTDfloat@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #float ).
	100 = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test42
	| api mangled k |
	k := 100.2.
	mangled := #intCfloat.
	api := self library #get: mangled.
	api := self cdecl: api return: #int32 types: #( #float ).
	100 = (# #api: k) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test43
	| api mangled k result |
	k := -1002.34.
	mangled := '_doubleSTDfloat@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #double types: #( #float ).
	result := # #api: k.
	(result * 100) rounded / 100 = k ifFalse: [ ^'Failed ',mangled ,' k=',k,' result=',result ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test44
	| api mangled k result |
	k := -1002.34.
	mangled := #doubleCfloat.
	api := self library #get: mangled.
	api := self cdecl: api return: #double types: #( #float ).
	result := # #api: k.
	(result * 100) rounded / 100 = k ifFalse: [ ^'Failed ',mangled ,' k=',k,' result=',result ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test45
	| api mangled k result |
	k := -12.34.
	mangled := '_floatSTDdouble@8'.
	api := self library #get: mangled.
	api := self stdCall: api return: #float types: #( #double ).
	result := # #api: k.
	(result * 100) rounded / 100 = k ifFalse: [ ^'Failed ',mangled ,' k=',k,' result=',result ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test46
	| api mangled k result |
	k := -12.34.
	mangled := #floatCdouble.
	api := self library #get: mangled.
	api := self cdecl: api return: #float types: #( #double ).
	result := # #api: k.
	(result * 100) rounded / 100 = k ifFalse: [ ^'Failed ',mangled ,' k=',k,' result=',result ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test47
	| api mangled |
	mangled := '_floatSTDfloat40@160'.
	api := self library #get: mangled.
	api := self
		stdCall: api return: #float
		types: #(
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
		).
	40 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test48
	| api mangled |
	mangled := '_floatSTDfloat80@320'.
	api := self library #get: mangled.
	api := self
		stdCall: api return: #float
		types: #(
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
		).
	80 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test49
	| api mangled |
	mangled := #floatCfloat40.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #float
		types: #(
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
		).
	40 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test50
	| api mangled |
	mangled := #floatCfloat80.
	api := self library #get: mangled.
	api := self
		cdecl: api return: #float
		types: #(
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
			#float #float #float #float #float
		).
	80 = (# #api: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		with: 1 with: 1 with: 1 with: 1 with: 1
		) ifFalse: [ ^'Failed ',mangled ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test51
	| api mangled |
	mangled := '_voidSTDvoidException@0'.
	api := self library #get: mangled.
	api := self stdCall: api return: #void types: #().
true ifTrue: [ ^'Failed - Can''t handle thrown C exception. ' ].
	self throws: [ # #api ].
	^#Ok! !

! TestDLL class methodsFor: 'tests-api' !
test52
	| api mangled |
	mangled := #voidCvoidException.
	api := self library #get: mangled.
	api := self cdecl: api return: #void types: #().
true ifTrue: [ ^'Failed - Can''t handle thrown C exception. ' ].
	self throws: [ # #api ].
	^#Ok! !


! TestDLL class methodsFor: 'utilities-callbacks' !
callback: argCount abi: abi doing: aBlock
	| callback types |
	types := (1 to: argCount) collect: [:i| #int32 ].
	^self
		new: #Callback
		with: (Array with: #int32 with: types)
		with: aBlock
		with: abi! !

! TestDLL class methodsFor: 'utilities-callbacks' !
stdCallback: argCount doing: aBlock
	^self callback: argCount abi: 2 doing: aBlock! !

! TestDLL class methodsFor: 'utilities-callbacks' !
cCallback: argCount doing: aBlock
	^self callback: argCount abi: 1 doing: aBlock! !

! TestDLL class methodsFor: 'tests' !
test53
	| callback api mangled |
	callback := self stdCallback: 0 doing: [
		self vprint: '	Executing std callback0'.
		1
	].
	mangled := '_callMeSTD0@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #pointer ).
	result := # #api: callback getPointer.
	result = 1 ifFalse: [ ^'Failed ',mangled ,' result=',result ].
	^#Ok! !

! TestDLL class methodsFor: 'tests' !
test54
	| callback api mangled |
	callback := self stdCallback: 1 doing: [:value|
		self vprint: '	Executing std callback1 with ',value.
		1
	].
	mangled := '_callMeSTD1@8'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #pointer ).
	result := # #api: callback getPointer.
	result = 1 ifFalse: [ ^'Failed ',mangled ,' result=',result ].
	^#Ok! !

! TestDLL class methodsFor: 'tests' !
test55
	| callback api mangled |
	callback := self cCallback: 0 doing: [
		self vprint: '	Executing c callback0'.
		1
	].
	mangled := '_callMeC0@4'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #pointer ).
	result := # #api: callback getPointer.
	result = 1 ifFalse: [ ^'Failed ',mangled ,' result=',result ].
	^#Ok! !

! TestDLL class methodsFor: 'tests' !
test56
	| callback api mangled |
	callback := self cCallback: 1 doing: [:value|
		self vprint: '	Executing c callback1 with ',value.
		1
	].
	mangled := '_callMeC1@8'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types: #( #pointer ).
	result := # #api: callback getPointer.
	result = 1 ifFalse: [ ^'Failed ',mangled ,' result=',result ].
	^#Ok! !

! TestDLL class methodsFor: 'tests' !
test57
	| callbacks args api mangled |
	args := (1 to: 10) asArray.
	callbacks := (1 to: 10) collect: [:i|
		self stdCallback: i doing: [
			self vprint: '	Executing std callback',i.
			1
		].
	].
	mangled := '_callMeSTD@84'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types:
		#( #int32 )
		, (callbacks collect: [:each| #pointer ])
		, (args collect: [:each| #int32 ]).
	(0 to: 10) do: [:count|
		self gc.
		result := api #apply: self with:
			(Array with: count)
			, (callbacks collect: [:each| each getPointer ])
			, args.
		result = 1 ifFalse: [ ^'Failed ',mangled ,' result=',result ].
	].
	^#Ok! !

! TestDLL class methodsFor: 'tests' !
test58
	| callbacks args api mangled |
	args := (1 to: 10) asArray.
	callbacks := (1 to: 10) collect: [:i|
		self cCallback: i doing: [
			self vprint: '	Executing c callback',i.
			1
		].
	].
	mangled := '_callMeC@84'.
	api := self library #get: mangled.
	api := self stdCall: api return: #int32 types:
		#( #int32 )
		, (callbacks collect: [:each| #pointer ])
		, (args collect: [:each| #int32 ]).
	(0 to: 10) do: [:count|
		self gc.
		result := api #apply: self with:
			(Array with: count)
			, (callbacks collect: [:each| each getPointer ])
			, args.
		result = 1 ifFalse: [ ^'Failed ',mangled ,' result=',result ].
	].
	^#Ok! !

! TestDLL class methodsFor: 'tests' !
test59
	| api mangled none ptr |
	ptr := self newPointer: 1024.
	ptr #putCString: 'ReadMe.txt'.
	none := {'null'}.
	mangled := '_MySearchPath@24'.
	api := self library #get: mangled.
	api := self stdCall: api return: #uint32 types:
		#( #pointer #pointer #pointer #uint32 #pointer #pointer ).
	result := # #api: none
		file: ptr
		extension: none
		bufSize: 0 buffer: none
		filePart: none.
	^#Ok ,' - Result =', result! !


self print: '//--ToDo: more tests --------------------------------------'.!
!self cancelFileIn!

